<?php
namespace WoocommercePpl\Admin\RestController;
defined("WPINC") or die();


use PluginPpl\MyApi2\Model\EpsApiInfrastructureWebApiModelProblemJsonModel;
use WoocommercePpl\Admin\CPLOperation;
use WoocommercePpl\Data\ShipmentData;
use WoocommercePpl\Admin\Errors;
use WoocommercePpl\Admin\RestResponse\RestResponse400;
use WoocommercePpl\Model\Model\CreateShipmentLabelBatchModel;
use WoocommercePpl\Model\Model\PrepareShipmentBatchModel;
use WoocommercePpl\Model\Model\PrepareShipmentBatchReturnModel;
use WoocommercePpl\Model\Model\RefreshShipmentBatchReturnModel;
use WoocommercePpl\Model\Model\ShipmentLabelRefreshBatchModel;
use WoocommercePpl\Model\Model\ShipmentModel;
use WoocommercePpl\Serializer;
use WoocommercePpl\Traits\ParcelDataModelTrait;
use WoocommercePpl\Validator\Validator;

class ShipmentBatchV1Controller extends  PPLRestController
{
    use ParcelDataModelTrait;

    protected $namespace = "woocommerce-ppl/v1";

    protected $base = "shipment/batch";

    public function register_routes()
    {

        register_rest_route($this->namespace, "/" . $this->base . "/preparing", [
            "methods" => \WP_REST_Server::EDITABLE,
            "callback" => [$this, "prepare_shipments"],
            "permission_callback"=>[$this, "check_permission"],
        ]);

        register_rest_route($this->namespace, "/" . $this->base . "/create-labels", [
            "methods" => \WP_REST_Server::EDITABLE,
            "callback" => [$this, "create_labels"],
            "permission_callback"=>[$this, "check_permission"],
        ]);

        register_rest_route($this->namespace, "/" . $this->base . "/refresh-labels", [
            "methods" => \WP_REST_Server::EDITABLE,
            "callback" => [$this, "refresh_labels"],
            "permission_callback"=>[$this, "check_permission"],
        ]);
    }

    public function create_labels(\WP_REST_Request $request)
    {
        $data = $request->get_json_params();

        /**
         * @var CreateShipmentLabelBatchModel $data
         */
        $data = Serializer::getInstance()->denormalize($data, CreateShipmentLabelBatchModel::class);
        $error = new Errors();

        $cpl = new CPLOperation();
        $resp = new \WP_REST_Response();
        $resp->set_status(204);

        try {
            $cpl->createPackages($data->getShipmentId());
        }
        catch (\Exception $exception)
        {
            if ($exception instanceof EpsApiInfrastructureWebApiModelProblemJsonModel) {
                if ($exception->getStatus() === 400) {
                    $resp->set_status(400);
                }
            } else {
                $resp->set_status(500);
                return $resp;
            }
        }
        $output = [];
        foreach ($data->getShipmentId() as $id) {
            $item = new ShipmentData($id);
            $item = Serializer::getInstance()->denormalize($item, ShipmentModel::class);
            $item = Serializer::getInstance()->normalize($item, "array");
            $output[] = $item;
        }
        $resp->set_data($output);

        return $resp;
    }

    public function prepare_shipments(\WP_REST_Request $request)
    {
        $data = $request->get_json_params();

        /**
         * @var PrepareShipmentBatchModel $data
         */
        $data = Serializer::getInstance()->denormalize($data, PrepareShipmentBatchModel::class);
        $error = new Errors();

        foreach ($data->getItems() as $key => $item) {
            if ($item->getShipmentId())
            {
                $shipmentData = new ShipmentData($item->getShipmentId());

                if (!$shipmentData->get_import_state() || $shipmentData->get_import_state() === "None")
                {
                    /**
                     * @var ShipmentModel $shipmentModel
                     */
                    $shipmentModel = Serializer::getInstance()->denormalize($shipmentData, ShipmentModel::class);
                    Validator::getInstance()->validate($shipmentModel, $error, "item.$key");
                }
            }
            else if ($item->getOrderId())
            {
                $order = new \WC_Order($item->getOrderId());
                if (self::hasPPLShipment($order)) {
                    $shipmentModel = Serializer::getInstance()->denormalize($order, ShipmentModel::class);
                    Validator::getInstance()->validate($shipmentModel, $error, "item.$key");

                } else {
                    $error->add("item.$key", "Nelze automaticky vytvořit zásilku");
                }
            } else {
                $error->add("item.$key", "Nelze automaticky vytvořit zásilku");
            }
        }

        if ($error->errors) {
            $resp = new RestResponse400($error);
            return $resp;
        }

        $output = [];

        foreach ($data->getItems() as $key => $item) {
            if ($item->getShipmentId()) {
                $output[$key] = $item->getShipmentId();
            }
            else if ($item->getOrderId())
            {
                $order = new \WC_Order($item->getOrderId());
                $shipmentModel = Serializer::getInstance()->denormalize($order, ShipmentModel::class);
                $shipmentData = Serializer::getInstance()->denormalize($shipmentModel, ShipmentData::class);
                $shipmentData->save();
                $output[$key] = $shipmentData->get_id();
            }
        }

        $model = new PrepareShipmentBatchReturnModel();
        $model->setShipmentId($output);

        $resp = new \WP_REST_Response();
        $resp->set_data(Serializer::getInstance()->normalize($model, "array"));
        return $resp;
    }

    public function refresh_labels(\WP_REST_Request $request)
    {
        $data = $request->get_json_params();

        /**
         * @var ShipmentLabelRefreshBatchModel $data
         */
        $data = Serializer::getInstance()->denormalize($data, ShipmentLabelRefreshBatchModel::class);
        $output = [];

        $batchs = [];

        foreach ($data->getShipmentId() as $item) {
            $shipmentData = new ShipmentData($item);
            $batchs[] = $shipmentData->get_batch_id();
        }

        $batchs = array_unique($batchs);
        $ids = [];
        $operations = new CPLOperation();
        $operations->loadingShipmentNumbers($batchs);

        foreach ($data->getShipmentId() as $item) {
            $shipmentData = new ShipmentData($item);
            $output[] = Serializer::getInstance()->denormalize($shipmentData, ShipmentModel::class);
            $ids[] = $shipmentData->get_batch_label_group();
        }

        $refresh = new RefreshShipmentBatchReturnModel();
        $refresh->setShipments($output);
        $refresh->setBatchs(array_filter($ids));
        $data = Serializer::getInstance()->normalize($refresh, "array");
        $resp = new \WP_REST_Response();
        $resp->set_data($data);

        return $resp;
    }
}