<?php
namespace WoocommercePpl\Admin\RestController;
defined("WPINC") or die();


use WoocommercePpl\Admin\CPLOperation;
use WoocommercePpl\Data\ParcelData;
use WoocommercePpl\Data\ShipmentData;
use WoocommercePpl\Admin\Errors;
use WoocommercePpl\Admin\Operation\FindParcelShopOperation;
use WoocommercePpl\Admin\RestResponse\RestResponse400;
use WoocommercePpl\Model\Model\BankAccountModel;
use WoocommercePpl\Model\Model\RecipientAddressModel;
use WoocommercePpl\Model\Model\ShipmentModel;
use WoocommercePpl\Model\Model\UpdateShipmentBankAccountModel;
use WoocommercePpl\Model\Model\UpdateShipmentModel;
use WoocommercePpl\Model\Model\UpdateShipmentParcelModel;
use WoocommercePpl\Model\Model\UpdateShipmentSender;
use WoocommercePpl\Model\Model\UpdateShipmentSenderModel;
use WoocommercePpl\Serializer;
use WoocommercePpl\Validator\Validator;

class ShipmentV1RestController extends PPLRestController
{
    protected $namespace = "woocommerce-ppl/v1";

    protected $base = "shipment";

    public function register_routes()
    {
        register_rest_route($this->namespace, '/' . $this->base. "/(?P<id>\d+)", [
            "methods" => \WP_REST_Server::READABLE,
            "callback" => [$this, "get_shipment"],
            "permission_callback"=>[$this, "check_permission"],
            "args"=> [
                "id" => [
                    "validate_callback" => function($params, $request, $key) {
                        return is_numeric($params) && (new ShipmentData($params))->get_id();
                    }
                ]
            ]
        ]);

        register_rest_route($this->namespace, "/" . $this->base . "/(?P<id>\d+)/recipient",
        [
            "methods" => \WP_REST_Server::EDITABLE,
            "callback" => [$this, "update_recipient_address"],
            "permission_callback"=>[$this, "check_permission"],
            "args"=> [
                "id" => [
                    "validate_callback" => function($params, $request, $key) {
                        return is_numeric($params) && (new ShipmentData($params))->get_id();
                    }
                ]
            ]
        ]);

        register_rest_route($this->namespace, "/" . $this->base,
            [
                "methods" => \WP_REST_Server::EDITABLE,
                "callback" => [$this, "create_shipment"],
                "permission_callback"=>[$this, "check_permission"],
            ]);

        register_rest_route($this->namespace, "/" . $this->base . "/(?P<id>\d+)",
            [
                "methods" => \WP_REST_Server::EDITABLE,
                "callback" => [$this, "update_shipment"],
                "permission_callback"=>[$this, "check_permission"],
                "args"=> [
                    "id" => [
                        "validate_callback" => function($params, $request, $key) {
                            return is_numeric($params) && (new ShipmentData($params))->get_id();
                        }
                    ]
                ]
            ]);

        register_rest_route($this->namespace, "/" . $this->base . "/(?P<id>\d+)/recipient",
            [
                "methods" => \WP_REST_Server::EDITABLE,
                "callback" => [$this, "update_shipment_recipient"],
                "permission_callback"=>[$this, "check_permission"],
                "args"=> [
                    "id" => [
                        "validate_callback" => function($params, $request, $key) {
                            return is_numeric($params) && (new ShipmentData($params))->get_id();
                        }
                    ]
                ]
            ]);

        register_rest_route($this->namespace, "/" . $this->base . "/(?P<id>\d+)/sender",
            [
                "methods" => \WP_REST_Server::EDITABLE,
                "callback" => [$this, "update_shipment_sender"],
                "permission_callback"=>[$this, "check_permission"],
                "args"=> [
                    "id" => [
                        "validate_callback" => function($params, $request, $key) {
                            return is_numeric($params) && (new ShipmentData($params))->get_id();
                        }
                    ]
                ]
            ]);

        register_rest_route($this->namespace, "/" . $this->base . "/(?P<id>\d+)/bankAccount",
            [
                "methods" => \WP_REST_Server::EDITABLE,
                "callback" => [$this, "update_shipment_bank_account"],
                "permission_callback"=>[$this, "check_permission"],
                "args"=> [
                    "id" => [
                        "validate_callback" => function($params, $request, $key) {
                            return is_numeric($params) && (new ShipmentData($params))->get_id();
                        }
                    ]
                ]
            ]);

        register_rest_route($this->namespace, "/" . $this->base . "/(?P<id>\d+)/parcel",
            [
                "methods" => \WP_REST_Server::EDITABLE,
                "callback" => [$this, "update_shipment_parcel"],
                "permission_callback"=>[$this, "check_permission"],
                "args"=> [
                    "id" => [
                        "validate_callback" => function($params, $request, $key) {
                            return is_numeric($params) && (new ShipmentData($params))->get_id();
                        }
                    ]
                ]
            ]);


    }

    public function get_shipment(\WP_REST_Request $request)
    {
        $shipmentData = new ShipmentData($request->get_param("id"));
        $shipmentModel = Serializer::getInstance()->denormalize($shipmentData, ShipmentModel::class);
        $data = Serializer::getInstance()->normalize($shipmentModel, "array");
        $response = new \WP_REST_Response();
        $response->set_data($data);
        return $response;
    }

    public function create_shipment(\WP_REST_Request $request)
    {
        $data = $request->get_json_params();
        $shipmentModel = Serializer::getInstance()->denormalize($data, UpdateShipmentModel::class);
        $shipment = Serializer::getInstance()->denormalize($shipmentModel, ShipmentData::class);
        $shipment->save();

        $resp = new \WP_REST_Response();
        $resp->set_status(201);
        $resp->set_headers([
            "Location" => rtrim(get_rest_url(), '/') . "/woocommerce-ppl/v1/shipment/{$shipment->get_id()}"
        ]);
        return $resp;

    }

    private function getShipment($id)
    {
        $shipment = new ShipmentData($id);
        if ($shipment->get_lock()) {
            try {
                $shipment->unlock();
            } catch (\Exception $ex) {
                $err = new \WP_Error();
                $err->add("locked", "Shipment is locked");
                $error = new RestResponse400($err);
                return $error;
            }
        }
        return $shipment;
    }

    public function update_recipient_address(\WP_REST_Request $request)
    {
        $data = $request->get_json_params();
        $sender = Serializer::getInstance()->denormalize($data, RecipientAddressModel::class);


        $validator = Validator::getInstance();
        $errors = new Errors();
        $validator->validate($sender, $errors, "");
        if ($errors->errors)
            return new RestResponse400($errors);


        $shipment = $this->getShipment($request->get_param("id"));
        if ($shipment instanceof \WP_REST_Response)
            return $shipment;
        $shipment = Serializer::getInstance()->denormalize($sender, ShipmentData::class, null, ["data" => $shipment]);
        $shipment->set_import_errors(null);
        $shipment->save();
        $resp = new \WP_REST_Response();
        $resp->set_status(204);
        return $resp;
    }

    public function update_shipment_bank_account(\WP_REST_Request $request)
    {
        $data = $request->get_json_params();
        /**
         * @var UpdateShipmentBankAccountModel $sender
         */
        $sender = Serializer::getInstance()->denormalize($data, UpdateShipmentBankAccountModel::class);
        $shipment = $this->getShipment($request->get_param("id"));

        if ($shipment instanceof \WP_REST_Response)
            return $shipment;
        $shipment->set_cod_bank_account_id($sender->getBankAccountId());
        $shipment->save();
        $resp = new \WP_REST_Response();
        $resp->set_status(204);
        return $resp;

    }

    public function update_shipment_parcel(\WP_REST_Request $request)
    {
        $data = $request->get_json_params();
        $sender = Serializer::getInstance()->denormalize($data, UpdateShipmentParcelModel::class);
        $shipment = $this->getShipment($request->get_param("id"));

        $founded = ParcelData::getAccessPointByCode($sender->getParcelCode());
        if (!$founded) {
            $find = new CPLOperation();
            $esp = $find->findParcel($sender->getParcelCode());
            if (!$esp) {
                $resp = new \WP_REST_Response();
                $resp->set_status(404);
                $resp->set_data("Not found accessPoint");
                return $resp;
            }
        }

        if ($shipment instanceof \WP_REST_Response)
            return $shipment;
        if (isset($esp))
            $parcel = Serializer::getInstance()->denormalize($esp, ParcelData::class, null, ["data" => $founded]);
        else
            $parcel = $founded;
        $parcel->save();
        $shipment->set_parcel_id($parcel->get_id());
        $shipment->save();

        $resp = new \WP_REST_Response();
        $resp->set_status(204);
        return $resp;
    }


    public function update_shipment_sender(\WP_REST_Request $request)
    {
        $data = $request->get_json_params();
        /**
         * @var UpdateShipmentSenderModel $sender
         */
        $sender = Serializer::getInstance()->denormalize($data, UpdateShipmentSenderModel::class);
        $shipment = $this->getShipment($request->get_param("id"));
        if ($shipment instanceof \WP_REST_Response)
            return $shipment;
        $shipment->set_sender_address_id($sender->getSenderId());
        $shipment->save();
        $resp = new \WP_REST_Response();
        $resp->set_status(204);
        return $resp;
    }

    public function update_shipment(\WP_REST_Request $request)
    {
        $data = $request->get_json_params();

        $shipmentModel = Serializer::getInstance()->denormalize($data, UpdateShipmentModel::class);

        $shipment = $this->getShipment($request->get_param("id"));

        if ($shipment instanceof \WP_REST_Response)
            return $shipment;

        $err = new Errors();
        Validator::getInstance()->validate($shipmentModel, $err, "");

        if ($err->errors)
            return new RestResponse400($err);


        $shipment = Serializer::getInstance()->denormalize($shipmentModel, ShipmentData::class, null, ["data" => $shipment]);
        $shipment->set_import_errors(null);
        $shipment->save();

        $resp = new \WP_REST_Response();
        $resp->set_status(204);
        return $resp;
    }


}